@@ -1,5 +1,5 @@
 /*
- * $Id: engine.c 5874 2011-11-18 08:40:54Z matthijs $
+ * $Id: engine.c 6162 2012-02-13 12:33:26Z jerry $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -41,6 +41,7 @@
 #include "scheduler/task.h"
 #include "shared/allocator.h"
 #include "shared/file.h"
+#include "shared/hsm.h"
 #include "shared/locks.h"
 #include "shared/log.h"
 #include "shared/privdrop.h"
@@ -308,7 +309,7 @@ engine_start_workers(engine_type* engine)
     }
     return;
 }
-static void
+void
 engine_start_drudgers(engine_type* engine)
 {
     size_t i = 0;
@@ -337,6 +338,7 @@ engine_stop_workers(engine_type* engine)
         engine->workers[i]->need_to_exit = 1;
         worker_wakeup(engine->workers[i]);
     }
+    worker_notify_all(&engine->signq->q_lock, &engine->signq->q_nonfull);
     /* head count */
     for (i=0; i < (size_t) engine->config->num_worker_threads; i++) {
         ods_log_debug("[%s] join worker %i", engine_str, i+1);
@@ -345,7 +347,7 @@ engine_stop_workers(engine_type* engine)
     }
     return;
 }
-static void
+void
 engine_stop_drudgers(engine_type* engine)
 {
     size_t i = 0;
@@ -657,15 +659,8 @@ engine_setup(engine_type* engine)
     sigaction(SIGTERM, &action, NULL);
 
     /* set up hsm */ /* LEAK */
-    result = hsm_open(engine->config->cfg_filename, hsm_prompt_pin, NULL);
+    result = lhsm_open(engine->config->cfg_filename);
     if (result != HSM_OK) {
-        char *error =  hsm_get_error(NULL);
-        if (error != NULL) {
-            ods_log_error("[%s] %s", engine_str, error);
-            free(error);
-        }
-        ods_log_error("[%s] error initializing libhsm (errno %i)",
-            engine_str, result);
         return ODS_STATUS_HSM_ERR;
     }
 
@@ -773,6 +768,7 @@ engine_run(engine_type* engine, int single_run)
     ods_log_debug("[%s] signer halted", engine_str);
     engine_stop_drudgers(engine);
     engine_stop_workers(engine);
+    (void)lhsm_reopen(engine->config->cfg_filename);
     return;
 }
 
