@@ -1,5 +1,5 @@
 /*
- * $Id: hsm.c 5380 2011-08-09 21:00:32Z matthijs $
+ * $Id: hsm.c 6127 2012-02-02 09:53:23Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -31,12 +31,103 @@
  *
  */
 
+#include "daemon/engine.h"
 #include "shared/hsm.h"
 #include "shared/log.h"
 
 static const char* hsm_str = "hsm";
 
 
+/**
+ * Open HSM.
+ *
+ */
+int
+lhsm_open(const char* filename)
+{
+    int result = hsm_open(filename, hsm_prompt_pin, NULL);
+    if (result != HSM_OK) {
+        char* error =  hsm_get_error(NULL);
+        if (error != NULL) {
+            ods_log_error("[%s] %s", hsm_str, error);
+            free(error);
+        } else {
+            ods_log_crit("[%s] error opening libhsm (errno %i)", hsm_str,
+                result);
+        }
+        /* exit? */
+    } else {
+        ods_log_info("[%s] libhsm connection opened succesfully", hsm_str);
+    }
+    return result;
+}
+
+
+/**
+ * Reopen HSM.
+ *
+ */
+int
+lhsm_reopen(const char* filename)
+{
+    if (hsm_check_context(NULL) != HSM_OK) {
+        ods_log_warning("[%s] idle libhsm connection, trying to reopen",
+            hsm_str);
+        hsm_close();
+        return lhsm_open(filename);
+    }
+    return HSM_OK;
+}
+
+
+/**
+ * Clear key cache.
+ *
+ */
+static void
+lhsm_clear_key_cache(key_type* key)
+{
+    if (!key) {
+        return;
+    }
+    if (key->dnskey) {
+        ldns_rr_free(key->dnskey);
+        key->dnskey = NULL;
+    }
+    if (key->hsmkey) {
+        hsm_key_free(key->hsmkey);
+        key->hsmkey = NULL;
+    }
+    if (key->params) {
+        hsm_sign_params_free(key->params);
+        key->params = NULL;
+    }
+    return;
+}
+
+
+/**
+ * Check the HSM connection, reload engine if necessary.
+ *
+ */
+void
+lhsm_check_connection(void* engine)
+{
+    engine_type* e = (engine_type*) engine;
+    if (hsm_check_context(NULL) != HSM_OK) {
+        ods_log_warning("[%s] idle libhsm connection, trying to reopen",
+            hsm_str);
+        engine_stop_drudgers(e);
+        hsm_close();
+        (void)lhsm_open(e->config->cfg_filename);
+        engine_start_drudgers((engine_type*) engine);
+    } else {
+        ods_log_debug("[%s] libhsm connection ok", hsm_str);
+    }
+    return;
+}
+
+
 /**
  * Get key from one of the HSMs.
  *
@@ -54,6 +145,8 @@ lhsm_get_key(hsm_ctx_t* ctx, ldns_rdf* owner, key_type* key_id)
     ods_log_assert(owner);
     ods_log_assert(key_id);
 
+lhsm_key_start:
+
     /* set parameters */
     if (!key_id->params) {
         key_id->params = hsm_sign_params_new();
@@ -67,6 +160,9 @@ lhsm_get_key(hsm_ctx_t* ctx, ldns_rdf* owner, key_type* key_id)
             if (error) {
                 ods_log_error("[%s] %s", hsm_str, error);
                 free((void*)error);
+            } else {
+                lhsm_clear_key_cache(key_id);
+                goto lhsm_key_start;
             }
             ods_log_error("[%s] unable to get key: create params for key %s "
                 "failed", hsm_str, key_id->locator?key_id->locator:"(null)");
@@ -83,6 +179,9 @@ lhsm_get_key(hsm_ctx_t* ctx, ldns_rdf* owner, key_type* key_id)
         if (error) {
             ods_log_error("[%s] %s", hsm_str, error);
             free((void*)error);
+        } else {
+            lhsm_clear_key_cache(key_id);
+            goto lhsm_key_start;
         }
         /* could not find key */
         ods_log_error("[%s] unable to get key: key %s not found", hsm_str,
@@ -99,6 +198,9 @@ lhsm_get_key(hsm_ctx_t* ctx, ldns_rdf* owner, key_type* key_id)
         if (error) {
             ods_log_error("[%s] %s", hsm_str, error);
             free((void*)error);
+        } else {
+            lhsm_clear_key_cache(key_id);
+            goto lhsm_key_start;
         }
         ods_log_error("[%s] unable to get key: hsm failed to create dnskey",
             hsm_str);
@@ -108,6 +210,7 @@ lhsm_get_key(hsm_ctx_t* ctx, ldns_rdf* owner, key_type* key_id)
     return ODS_STATUS_OK;
 }
 
+
 /**
  * Get RRSIG from one of the HSMs, given a RRset and a key.
  *
@@ -132,6 +235,8 @@ lhsm_sign(hsm_ctx_t* ctx, ldns_rr_list* rrset, key_type* key_id,
     ods_log_assert(inception);
     ods_log_assert(expiration);
 
+lhsm_sign_start:
+
     if (!key_id->dnskey) {
         status = lhsm_get_key(ctx, owner, key_id);
         if (status != ODS_STATUS_OK) {
@@ -139,6 +244,9 @@ lhsm_sign(hsm_ctx_t* ctx, ldns_rr_list* rrset, key_type* key_id,
             if (error) {
                 ods_log_error("[%s] %s", hsm_str, error);
                 free((void*)error);
+            } else {
+                lhsm_clear_key_cache(key_id);
+                goto lhsm_sign_start;
             }
             ods_log_error("[%s] unable to sign: get key failed", hsm_str);
             return NULL;
@@ -166,7 +274,11 @@ lhsm_sign(hsm_ctx_t* ctx, ldns_rr_list* rrset, key_type* key_id,
         if (error) {
             ods_log_error("[%s] %s", hsm_str, error);
             free((void*)error);
+        } else {
+            lhsm_clear_key_cache(key_id);
+            goto lhsm_sign_start;
         }
+        ods_log_crit("[%s] error signing rrset with libhsm", hsm_str);
     }
     return result;
 }
