@@ -1,5 +1,5 @@
 /*
- * $Id: zone.c 5802 2011-10-19 14:07:31Z matthijs $
+ * $Id: zone.c 6109 2012-01-30 10:30:52Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -373,7 +373,7 @@ zone_load_signconf(zone_type* zone, task_id* tbs)
     signconf_type* signconf = NULL;
     ldns_rr_list* del = NULL;
     char* datestamp = NULL;
-    uint32_t ustamp = 0;
+    uint32_t ustamp;
     task_id denial_what;
     task_id keys_what;
     task_id what;
@@ -538,6 +538,7 @@ zone_publish_dnskeys(zone_type* zone, int recover)
         ttl = (uint32_t) duration2time(zone->signconf->dnskey_ttl);
     }
 
+    /* check connection here? */
     ctx = hsm_create_context();
     if (ctx == NULL) {
         ods_log_error("[%s] unable to publish dnskeys for zone %s: error "
@@ -604,8 +605,6 @@ ods_status
 zone_prepare_nsec3(zone_type* zone, int recover)
 {
     ldns_rr* nsec3params_rr = NULL;
-    domain_type* apex = NULL;
-    rrset_type* rrset = NULL;
     ods_status status = ODS_STATUS_OK;
 
     if (!zone) {
@@ -679,32 +678,9 @@ zone_prepare_nsec3(zone_type* zone, int recover)
         nsec3params_cleanup(zone->nsec3params);
         zone->nsec3params = NULL;
         ldns_rr_free(nsec3params_rr);
-    } else if (!recover) {
-        /* add ok, wipe out previous nsec3params */
-        apex = zonedata_lookup_domain(zone->zonedata, zone->dname);
-        if (!apex) {
-            ods_log_crit("[%s] unable to delete previous NSEC3PARAM RR "
-            "from zone %s: apex undefined", zone_str, zone->name);
-            nsec3params_cleanup(zone->nsec3params);
-            zone->nsec3params = NULL;
-            zonedata_rollback(zone->zonedata);
-            return ODS_STATUS_ASSERT_ERR;
-        }
-        ods_log_assert(apex);
-
-        rrset = domain_lookup_rrset(apex, LDNS_RR_TYPE_NSEC3PARAMS);
-        if (rrset) {
-            status = rrset_wipe_out(rrset);
-            if (status != ODS_STATUS_OK) {
-                ods_log_error("[%s] unable to wipe out previous "
-                    "NSEC3PARAM RR from zone %s", zone_str, zone->name);
-                nsec3params_cleanup(zone->nsec3params);
-                zone->nsec3params = NULL;
-                rrset_rollback(rrset);
-                return status;
-            }
-        }
     }
+    /* previous nsec3params is already withdrawn during load signconf */
+
     return status;
 }
 
@@ -953,6 +929,11 @@ zone_recover(zone_type* zone)
         zone->task = (void*) task;
         zone->signconf->last_modified = lastmod;
 
+        /**
+         * The function zone_publish_dnskeys() uses hsm_create_context().
+         * We don't have to check the hsm connection here, zone_recover()
+         * is part of engine_start() and is ran only once.
+         */
         status = zone_publish_dnskeys(zone, 1);
         if (status != ODS_STATUS_OK) {
             zone->task = NULL;
